@@ -13,7 +13,7 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 
-"""Implementation of the public test library logging API.
+"""Implementation of the public logging API for libraries.
 
 This is exposed via :py:mod:`robot.api.logger`. Implementation must reside
 here to avoid cyclic imports.
@@ -22,8 +22,7 @@ here to avoid cyclic imports.
 import sys
 import threading
 
-from robot.errors import DataError
-from robot.utils import unic, console_encode
+from robot.utils import console_encode
 
 from .logger import LOGGER
 from .loggerhelper import Message
@@ -37,10 +36,10 @@ def write(msg, level, html=False):
     # expose this functionality publicly. See the following issue for details:
     # https://github.com/robotframework/robotframework/issues/1505
     if callable(msg):
-        msg = unic(msg)
+        msg = str(msg)
     if level.upper() not in ('TRACE', 'DEBUG', 'INFO', 'HTML', 'WARN', 'ERROR'):
-        raise DataError("Invalid log level '%s'." % level)
-    if threading.currentThread().getName() in LOGGING_THREADS:
+        raise RuntimeError("Invalid log level '%s'." % level)
+    if threading.current_thread().name in LOGGING_THREADS:
         LOGGER.log_message(Message(msg, level, html))
 
 
@@ -67,7 +66,7 @@ def error(msg, html=False):
 
 
 def console(msg, newline=True, stream='stdout'):
-    msg = unic(msg)
+    msg = str(msg)
     if newline:
         msg += '\n'
     stream = sys.__stdout__ if stream.lower() != 'stderr' else sys.__stderr__
