@@ -13,58 +13,50 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 
-from __future__ import print_function
-
 import os
 import subprocess
 import sys
-if sys.platform.startswith('java'):
-    from java.awt import Toolkit, Robot, Rectangle
-    from javax.imageio import ImageIO
-    from java.io import File
-elif sys.platform == 'cli':
-    import clr
-    clr.AddReference('System.Windows.Forms')
-    clr.AddReference('System.Drawing')
-    from System.Drawing import Bitmap, Graphics, Imaging
-    from System.Windows.Forms import Screen
-else:
-    try:
-        import wx
-    except ImportError:
-        wx = None
-    try:
-        from gtk import gdk
-    except ImportError:
-        gdk = None
-    try:
-        from PIL import ImageGrab  # apparently available only on Windows
-    except ImportError:
-        ImageGrab = None
+
+try:
+    import wx
+except ImportError:
+    wx = None
+try:
+    from gtk import gdk
+except ImportError:
+    gdk = None
+try:
+    from PIL import ImageGrab  # apparently available only on Windows
+except ImportError:
+    ImageGrab = None
 
 from robot.api import logger
 from robot.libraries.BuiltIn import BuiltIn
 from robot.version import get_version
-from robot.utils import abspath, get_error_message, get_link_path, py2to3
+from robot.utils import abspath, get_error_message, get_link_path
 
 
-class Screenshot(object):
-    """Test library for taking screenshots on the machine where tests are run.
+class Screenshot:
+    """Library for taking screenshots on the machine where tests are executed.
 
-    Notice that successfully taking screenshots requires tests to be run with
-    a physical or virtual display.
+    Taking the actual screenshot requires a suitable tool or module that may
+    need to be installed separately. Taking screenshots also requires tests
+    to be run with a physical or virtual display.
 
-    = Using with Python =
+    == Table of contents ==
 
-    How screenshots are taken when using Python depends on the operating
-    system. On OSX screenshots are taken using the built-in ``screencapture``
-    utility. On other operating systems you need to have one of the following
-    tools or Python modules installed. You can specify the tool/module to use
-    when `importing` the library. If no tool or module is specified, the first
+    %TOC%
+
+    = Supported screenshot taking tools and modules =
+
+    How screenshots are taken depends on the operating system. On OSX
+    screenshots are taken using the built-in ``screencapture`` utility. On
+    other operating systems you need to have one of the following tools or
+    Python modules installed. You can specify the tool/module to use when
+    `importing` the library. If no tool or module is specified, the first
     one found will be used.
 
-    - wxPython :: http://wxpython.org :: Required also by RIDE so many Robot
-      Framework users already have this module installed.
+    - wxPython :: http://wxpython.org :: Generic Python GUI toolkit.
     - PyGTK :: http://pygtk.org :: This module is available by default on most
       Linux distributions.
     - Pillow :: http://python-pillow.github.io ::
@@ -72,16 +64,6 @@ class Screenshot(object):
     - Scrot :: http://en.wikipedia.org/wiki/Scrot :: Not used on Windows.
       Install with ``apt-get install scrot`` or similar.
 
-    Using ``screencapture`` on OSX and specifying explicit screenshot module
-    are new in Robot Framework 2.9.2. The support for using ``scrot`` is new
-    in Robot Framework 3.0.
-
-    = Using with Jython and IronPython =
-
-    With Jython and IronPython this library uses APIs provided by JVM and .NET
-    platforms, respectively. These APIs are always available and thus no
-    external modules are needed.
-
     = Where screenshots are saved =
 
     By default screenshots are saved into the same directory where the Robot
@@ -92,6 +74,13 @@ class Screenshot(object):
     ``screenshot_directory`` argument when `importing` the library and
     using `Set Screenshot Directory` keyword during execution. It is also
     possible to save screenshots using an absolute path.
+
+    = ScreenCapLibrary =
+
+    [https://github.com/mihaiparvu/ScreenCapLibrary|ScreenCapLibrary] is an
+    external Robot Framework library that can be used as an alternative,
+    which additionally provides support for multiple formats, adjusting the
+    quality, using GIFs and video capturing.
     """
 
     ROBOT_LIBRARY_SCOPE = 'TEST SUITE'
@@ -105,18 +94,15 @@ class Screenshot(object):
         `Set Screenshot Directory` keyword.
 
         ``screenshot_module`` specifies the module or tool to use when using
-        this library on Python outside OSX. Possible values are ``wxPython``,
+        this library outside OSX. Possible values are ``wxPython``,
         ``PyGTK``, ``PIL`` and ``scrot``, case-insensitively. If no value is
-        given, the first module/tool found is used in that order. See `Using
-        with Python` for more information.
+        given, the first module/tool found is used in that order.
 
-        Examples (use only one of these):
+        Examples:
         | =Setting= |  =Value=   |  =Value=   |
         | Library   | Screenshot |            |
         | Library   | Screenshot | ${TEMPDIR} |
         | Library   | Screenshot | screenshot_module=PyGTK |
-
-        Specifying explicit screenshot module is new in Robot Framework 2.9.2.
         """
         self._given_screenshot_dir = self._norm_path(screenshot_directory)
         self._screenshot_taker = ScreenshotTaker(screenshot_module)
@@ -124,7 +110,11 @@ class Screenshot(object):
     def _norm_path(self, path):
         if not path:
             return path
-        return os.path.normpath(path.replace('/', os.sep))
+        elif isinstance(path, os.PathLike):
+            path = str(path)
+        else:
+            path = path.replace('/', os.sep)
+        return os.path.normpath(path)
 
     @property
     def _screenshot_dir(self):
@@ -193,8 +183,9 @@ class Screenshot(object):
         self._link_screenshot(path)
         return path
 
-    def _save_screenshot(self, basename, directory=None):
-        path = self._get_screenshot_path(basename, directory)
+    def _save_screenshot(self, name):
+        name = str(name) if isinstance(name, os.PathLike) else name.replace('/', os.sep)
+        path = self._get_screenshot_path(name)
         return self._screenshot_to_file(path)
 
     def _screenshot_to_file(self, path):
@@ -216,14 +207,13 @@ class Screenshot(object):
                                "does not exist" % os.path.dirname(path))
         return path
 
-    def _get_screenshot_path(self, basename, directory):
-        directory = self._norm_path(directory) if directory else self._screenshot_dir
+    def _get_screenshot_path(self, basename):
         if basename.lower().endswith(('.jpg', '.jpeg')):
-            return os.path.join(directory, basename)
+            return os.path.join(self._screenshot_dir, basename)
         index = 0
         while True:
             index += 1
-            path = os.path.join(directory, "%s_%d.jpg" % (basename, index))
+            path = os.path.join(self._screenshot_dir, "%s_%d.jpg" % (basename, index))
             if not os.path.exists(path):
                 return path
 
@@ -238,8 +228,7 @@ class Screenshot(object):
                     % (link, path), html=True)
 
 
-@py2to3
-class ScreenshotTaker(object):
+class ScreenshotTaker:
 
     def __init__(self, module_name=None):
         self._screenshot = self._get_screenshot_taker(module_name)
@@ -249,7 +238,7 @@ class ScreenshotTaker(object):
     def __call__(self, path):
         self._screenshot(path)
 
-    def __nonzero__(self):
+    def __bool__(self):
         return self.module != 'no'
 
     def test(self, path=None):
@@ -271,10 +260,6 @@ class ScreenshotTaker(object):
             return True
 
     def _get_screenshot_taker(self, module_name=None):
-        if sys.platform.startswith('java'):
-            return self._java_screenshot
-        if sys.platform == 'cli':
-            return self._cli_screenshot
         if sys.platform == 'darwin':
             return self._osx_screenshot
         if module_name:
@@ -303,22 +288,6 @@ class ScreenshotTaker(object):
             if module:
                 return screenshot_taker
 
-    def _java_screenshot(self, path):
-        size = Toolkit.getDefaultToolkit().getScreenSize()
-        rectangle = Rectangle(0, 0, size.width, size.height)
-        image = Robot().createScreenCapture(rectangle)
-        ImageIO.write(image, 'jpg', File(path))
-
-    def _cli_screenshot(self, path):
-        bmp = Bitmap(Screen.PrimaryScreen.Bounds.Width,
-                     Screen.PrimaryScreen.Bounds.Height)
-        graphics = Graphics.FromImage(bmp)
-        try:
-            graphics.CopyFromScreen(0, 0, 0, 0, bmp.Size)
-        finally:
-            graphics.Dispose()
-            bmp.Save(path, Imaging.ImageFormat.Jpeg)
-
     def _osx_screenshot(self, path):
         if self._call('screencapture', '-t', 'jpg', path) != 0:
             raise RuntimeError("Using 'screencapture' failed.")
@@ -338,6 +307,8 @@ class ScreenshotTaker(object):
         if not path.endswith(('.jpg', '.jpeg')):
             raise RuntimeError("Scrot requires extension to be '.jpg' or "
                                "'.jpeg', got '%s'." % os.path.splitext(path)[1])
+        if os.path.exists(path):
+            os.remove(path)
         if self._call('scrot', '--silent', path) != 0:
             raise RuntimeError("Using 'scrot' failed.")
 
@@ -378,7 +349,7 @@ class ScreenshotTaker(object):
 
 if __name__ == "__main__":
     if len(sys.argv) not in [2, 3]:
-        sys.exit("Usage: %s <path>|test [wx|pygtk|pil|scrot]"
+        sys.exit("Usage: %s <path>|test [wxpython|pygtk|pil|scrot]"
                  % os.path.basename(sys.argv[0]))
     path = sys.argv[1] if sys.argv[1] != 'test' else None
     module = sys.argv[2] if len(sys.argv) > 2 else None
