@@ -1,4 +1,5 @@
-#  Copyright 2008-2015 Nokia Solutions and Networks
+#  Copyright 2008-2015 Nokia Networks
+#  Copyright 2016-     Robot Framework Foundation
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,19 +13,21 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 
+import os
 import re
 from fnmatch import fnmatchcase
 from random import randint
 from string import ascii_lowercase, ascii_uppercase, digits
 
 from robot.api import logger
-from robot.utils import (is_bytes, is_string, is_truthy, is_unicode, lower,
-                         unic, PY3)
+from robot.api.deco import keyword
+from robot.utils import (FileReader, is_bytes, is_string, is_truthy,
+                         parse_re_flags, safe_str, type_name)
 from robot.version import get_version
 
 
-class String(object):
-    """A test library for string manipulation and verification.
+class String:
+    """A library for string manipulation and verification.
 
     ``String`` is Robot Framework's standard library for manipulating
     strings (e.g. `Replace String Using Regexp`, `Split To Lines`) and
@@ -47,34 +50,96 @@ class String(object):
     ROBOT_LIBRARY_SCOPE = 'GLOBAL'
     ROBOT_LIBRARY_VERSION = get_version()
 
-    def convert_to_lowercase(self, string):
-        """Converts string to lowercase.
+    def convert_to_lower_case(self, string):
+        """Converts string to lower case.
+
+        Uses Python's standard
+        [https://docs.python.org/library/stdtypes.html#str.lower|lower()]
+        method.
 
         Examples:
-        | ${str1} = | Convert To Lowercase | ABC |
-        | ${str2} = | Convert To Lowercase | 1A2c3D |
+        | ${str1} = | Convert To Lower Case | ABC |
+        | ${str2} = | Convert To Lower Case | 1A2c3D |
         | Should Be Equal | ${str1} | abc |
         | Should Be Equal | ${str2} | 1a2c3d |
-
-        New in Robot Framework 2.8.6.
         """
-        # Custom `lower` needed due to IronPython bug. See its code and
-        # comments for more details.
-        return lower(string)
+        return string.lower()
 
-    def convert_to_uppercase(self, string):
-        """Converts string to uppercase.
+    def convert_to_upper_case(self, string):
+        """Converts string to upper case.
+
+        Uses Python's standard
+        [https://docs.python.org/library/stdtypes.html#str.upper|upper()]
+        method.
 
         Examples:
-        | ${str1} = | Convert To Uppercase | abc |
-        | ${str2} = | Convert To Uppercase | 1a2C3d |
+        | ${str1} = | Convert To Upper Case | abc |
+        | ${str2} = | Convert To Upper Case | 1a2C3d |
         | Should Be Equal | ${str1} | ABC |
         | Should Be Equal | ${str2} | 1A2C3D |
-
-        New in Robot Framework 2.8.6.
         """
         return string.upper()
 
+    @keyword(types=None)
+    def convert_to_title_case(self, string, exclude=None):
+        """Converts string to title case.
+
+        Uses the following algorithm:
+
+        - Split the string to words from whitespace characters (spaces,
+          newlines, etc.).
+        - Exclude words that are not all lower case. This preserves,
+          for example, "OK" and "iPhone".
+        - Exclude also words listed in the optional ``exclude`` argument.
+        - Title case the first alphabetical character of each word that has
+          not been excluded.
+        - Join all words together so that original whitespace is preserved.
+
+        Explicitly excluded words can be given as a list or as a string with
+        words separated by a comma and an optional space. Excluded words are
+        actually considered to be regular expression patterns, so it is
+        possible to use something like "example[.!?]?" to match the word
+        "example" on it own and also if followed by ".", "!" or "?".
+        See `BuiltIn.Should Match Regexp` for more information about Python
+        regular expression syntax in general and how to use it in Robot
+        Framework data in particular.
+
+        Examples:
+        | ${str1} = | Convert To Title Case | hello, world!     |
+        | ${str2} = | Convert To Title Case | it's an OK iPhone | exclude=a, an, the |
+        | ${str3} = | Convert To Title Case | distance is 1 km. | exclude=is, km.? |
+        | Should Be Equal | ${str1} | Hello, World! |
+        | Should Be Equal | ${str2} | It's an OK iPhone |
+        | Should Be Equal | ${str3} | Distance is 1 km. |
+
+        The reason this keyword does not use Python's standard
+        [https://docs.python.org/library/stdtypes.html#str.title|title()]
+        method is that it can yield undesired results, for example, if
+        strings contain upper case letters or special characters like
+        apostrophes. It would, for example, convert "it's an OK iPhone"
+        to "It'S An Ok Iphone".
+
+        New in Robot Framework 3.2.
+        """
+        if not is_string(string):
+            raise TypeError('This keyword works only with Unicode strings.')
+        if is_string(exclude):
+            exclude = [e.strip() for e in exclude.split(',')]
+        elif not exclude:
+            exclude = []
+        exclude = [re.compile('^%s$' % e) for e in exclude]
+
+        def title(word):
+            if any(e.match(word) for e in exclude) or not word.islower():
+                return word
+            for index, char in enumerate(word):
+                if char.isalpha():
+                    return word[:index] + word[index].title() + word[index+1:]
+            return word
+
+        tokens = re.split(r'(\s+)', string, flags=re.UNICODE)
+        return ''.join(title(token) for token in tokens)
+
     def encode_string_to_bytes(self, string, encoding, errors='strict'):
         """Encodes the given Unicode ``string`` to bytes using the given ``encoding``.
 
@@ -95,8 +160,6 @@ class String(object):
         on character or integer sequences. Use `Decode Bytes To String` if you
         need to convert byte strings to Unicode strings and `Convert To String`
         in ``BuiltIn`` if you need to convert arbitrary objects to Unicode.
-
-        New in Robot Framework 2.7.7.
         """
         return bytes(string.encode(encoding, errors))
 
@@ -119,13 +182,43 @@ class String(object):
         Use `Encode String To Bytes` if you need to convert Unicode strings to
         byte strings, and `Convert To String` in ``BuiltIn`` if you need to
         convert arbitrary objects to Unicode strings.
-
-        New in Robot Framework 2.7.7.
         """
-        if PY3 and is_unicode(bytes):
-            raise TypeError('Can not decode strings on Python 3.')
+        if is_string(bytes):
+            raise TypeError('Cannot decode strings.')
         return bytes.decode(encoding, errors)
 
+    def format_string(self, template, *positional, **named):
+        """Formats a ``template`` using the given ``positional`` and ``named`` arguments.
+
+        The template can be either be a string or an absolute path to
+        an existing file. In the latter case the file is read and its contents
+        are used as the template. If the template file contains non-ASCII
+        characters, it must be encoded using UTF-8.
+
+        The template is formatted using Python's
+        [https://docs.python.org/library/string.html#format-string-syntax|format
+        string syntax]. Placeholders are marked using ``{}`` with possible
+        field name and format specification inside. Literal curly braces
+        can be inserted by doubling them like `{{` and `}}`.
+
+        Examples:
+        | ${to} = | Format String | To: {} <{}>                    | ${user}      | ${email} |
+        | ${to} = | Format String | To: {name} <{email}>           | name=${name} | email=${email} |
+        | ${to} = | Format String | To: {user.name} <{user.email}> | user=${user} |
+        | ${xx} = | Format String | {:*^30}                        | centered     |
+        | ${yy} = | Format String | {0:{width}{base}}              | ${42}        | base=X | width=10 |
+        | ${zz} = | Format String | ${CURDIR}/template.txt         | positional   | named=value |
+
+        New in Robot Framework 3.1.
+        """
+        if os.path.isabs(template) and os.path.isfile(template):
+            template = template.replace('/', os.sep)
+            logger.info('Reading template from file <a href="%s">%s</a>.'
+                        % (template, template), html=True)
+            with FileReader(template) as reader:
+                template = reader.read()
+        return template.format(*positional, **named)
+
     def get_line_count(self, string):
         """Returns and logs the number of lines in the given string."""
         count = len(string.splitlines())
@@ -184,8 +277,9 @@ class String(object):
 
         The match is case-sensitive by default, but giving ``case_insensitive``
         a true value makes it case-insensitive. The value is considered true
-        if it is a non-empty string that is not equal to ``false`` or ``no``.
-        If the value is not a string, its truth value is got directly in Python.
+        if it is a non-empty string that is not equal to ``false``, ``none`` or
+        ``no``. If the value is not a string, its truth value is got directly
+        in Python.
 
         Lines are returned as one string catenated back together with
         newlines. Possible trailing newline is never returned. The
@@ -218,8 +312,9 @@ class String(object):
 
         The match is case-sensitive by default, but giving ``case_insensitive``
         a true value makes it case-insensitive. The value is considered true
-        if it is a non-empty string that is not equal to ``false`` or ``no``.
-        If the value is not a string, its truth value is got directly in Python.
+        if it is a non-empty string that is not equal to ``false``, ``none`` or
+        ``no``. If the value is not a string, its truth value is got directly
+        in Python.
 
         Lines are returned as one string catenated back together with
         newlines. Possible trailing newline is never returned. The
@@ -240,24 +335,27 @@ class String(object):
             matches = lambda line: fnmatchcase(line, pattern)
         return self._get_matching_lines(string, matches)
 
-    def get_lines_matching_regexp(self, string, pattern, partial_match=False):
+    def get_lines_matching_regexp(self, string, pattern, partial_match=False, flags=None):
         """Returns lines of the given ``string`` that match the regexp ``pattern``.
 
         See `BuiltIn.Should Match Regexp` for more information about
         Python regular expression syntax in general and how to use it
-        in Robot Framework test data in particular.
+        in Robot Framework data in particular.
 
-        By default lines match only if they match the pattern fully, but
+        Lines match only if they match the pattern fully by default, but
         partial matching can be enabled by giving the ``partial_match``
-        argument a true value. The value is considered true if it is a
-        non-empty string that is not equal to ``false`` or ``no``. If the
-        value is not a string, its truth value is got directly in Python.
+        argument a true value. The value is considered true
+        if it is a non-empty string that is not equal to ``false``, ``none`` or
+        ``no``. If the value is not a string, its truth value is got directly
+        in Python.
 
         If the pattern is empty, it matches only empty lines by default.
         When partial matching is enabled, empty pattern matches all lines.
 
-        Notice that to make the match case-insensitive, you need to prefix
-        the pattern with case-insensitive flag ``(?i)``.
+        Possible flags altering how the expression is parsed (e.g. ``re.IGNORECASE``,
+        ``re.VERBOSE``) can be given using the ``flags`` argument (e.g.
+        ``flags=IGNORECASE | VERBOSE``) or embedded to the pattern (e.g.
+        ``(?ix)pattern``).
 
         Lines are returned as one string concatenated back together with
         newlines. Possible trailing newline is never returned. The
@@ -267,17 +365,18 @@ class String(object):
         | ${lines} = | Get Lines Matching Regexp | ${result} | Reg\\\\w{3} example |
         | ${lines} = | Get Lines Matching Regexp | ${result} | Reg\\\\w{3} example | partial_match=true |
         | ${ret} =   | Get Lines Matching Regexp | ${ret}    | (?i)FAIL: .* |
+        | ${ret} =   | Get Lines Matching Regexp | ${ret}    | FAIL: .* | flags=IGNORECASE |
 
-        See `Get Lines Matching Pattern` and `Get Lines Containing
-        String` if you do not need full regular expression powers (and
-        complexity).
+        See `Get Lines Matching Pattern` and `Get Lines Containing String` if you
+        do not need the full regular expression powers (and complexity).
 
-        ``partial_match`` argument is new in Robot Framework 2.9. In earlier
-         versions exact match was always required.
+        The ``flags`` argument is new in Robot Framework 6.0.
         """
-        if not is_truthy(partial_match):
-            pattern = '^%s$' % pattern
-        return self._get_matching_lines(string, re.compile(pattern).search)
+        if is_truthy(partial_match):
+            match = re.compile(pattern, flags=parse_re_flags(flags)).search
+        else:
+            match = re.compile(pattern + '$', flags=parse_re_flags(flags)).match
+        return self._get_matching_lines(string, match)
 
     def _get_matching_lines(self, string, matches):
         lines = string.splitlines()
@@ -285,13 +384,13 @@ class String(object):
         logger.info('%d out of %d lines matched' % (len(matching), len(lines)))
         return '\n'.join(matching)
 
-    def get_regexp_matches(self, string, pattern, *groups):
+    def get_regexp_matches(self, string, pattern, *groups, flags=None):
         """Returns a list of all non-overlapping matches in the given string.
 
         ``string`` is the string to find matches from and ``pattern`` is the
         regular expression. See `BuiltIn.Should Match Regexp` for more
         information about Python regular expression syntax in general and how
-        to use it in Robot Framework test data in particular.
+        to use it in Robot Framework data in particular.
 
         If no groups are used, the returned list contains full matches. If one
         group is used, the list contains only contents of that group. If
@@ -299,9 +398,15 @@ class String(object):
         individual group contents. All groups can be given as indexes (starting
         from 1) and named groups also as names.
 
+        Possible flags altering how the expression is parsed (e.g. ``re.IGNORECASE``,
+        ``re.MULTILINE``) can be given using the ``flags`` argument (e.g.
+        ``flags=IGNORECASE | MULTILINE``) or embedded to the pattern (e.g.
+        ``(?im)pattern``).
+
         Examples:
         | ${no match} =    | Get Regexp Matches | the string | xxx     |
         | ${matches} =     | Get Regexp Matches | the string | t..     |
+        | ${matches} =     | Get Regexp Matches | the string | T..     | flags=IGNORECASE |
         | ${one group} =   | Get Regexp Matches | the string | t(..)   | 1 |
         | ${named group} = | Get Regexp Matches | the string | t(?P<name>..) | name |
         | ${two groups} =  | Get Regexp Matches | the string | t(.)(.) | 1 | 2 |
@@ -312,9 +417,9 @@ class String(object):
         | ${named group} = ['he', 'ri']
         | ${two groups} = [('h', 'e'), ('r', 'i')]
 
-        New in Robot Framework 2.9.
+        The ``flags`` argument is new in Robot Framework 6.0.
         """
-        regexp = re.compile(pattern)
+        regexp = re.compile(pattern, flags=parse_re_flags(flags))
         groups = [self._parse_group(g) for g in groups]
         return [m.group(*groups) for m in regexp.finditer(string)]
 
@@ -348,26 +453,33 @@ class String(object):
         count = self._convert_to_integer(count, 'count')
         return string.replace(search_for, replace_with, count)
 
-    def replace_string_using_regexp(self, string, pattern, replace_with, count=-1):
+    def replace_string_using_regexp(self, string, pattern, replace_with, count=-1, flags=None):
         """Replaces ``pattern`` in the given ``string`` with ``replace_with``.
 
         This keyword is otherwise identical to `Replace String`, but
         the ``pattern`` to search for is considered to be a regular
         expression.  See `BuiltIn.Should Match Regexp` for more
         information about Python regular expression syntax in general
-        and how to use it in Robot Framework test data in particular.
+        and how to use it in Robot Framework data in particular.
+
+        Possible flags altering how the expression is parsed (e.g. ``re.IGNORECASE``,
+        ``re.MULTILINE``) can be given using the ``flags`` argument (e.g.
+        ``flags=IGNORECASE | MULTILINE``) or embedded to the pattern (e.g.
+        ``(?im)pattern``).
 
         If you need to just remove a string see `Remove String Using Regexp`.
 
         Examples:
         | ${str} = | Replace String Using Regexp | ${str} | 20\\\\d\\\\d-\\\\d\\\\d-\\\\d\\\\d | <DATE> |
         | ${str} = | Replace String Using Regexp | ${str} | (Hello|Hi) | ${EMPTY} | count=1 |
+
+        The ``flags`` argument is new in Robot Framework 6.0.
         """
         count = self._convert_to_integer(count, 'count')
         # re.sub handles 0 and negative counts differently than string.replace
         if count == 0:
             return string
-        return re.sub(pattern, replace_with, string, max(count, 0))
+        return re.sub(pattern, replace_with, string, max(count, 0), flags=parse_re_flags(flags))
 
     def remove_string(self, string, *removables):
         """Removes all ``removables`` from the given ``string``.
@@ -388,14 +500,12 @@ class String(object):
         | Should Be Equal | ${str}        | Robot Frame     |
         | ${str} =        | Remove String | Robot Framework | o | bt |
         | Should Be Equal | ${str}        | R Framewrk      |
-
-        New in Robot Framework 2.8.2.
         """
         for removable in removables:
             string = self.replace_string(string, removable, '')
         return string
 
-    def remove_string_using_regexp(self, string, *patterns):
+    def remove_string_using_regexp(self, string, *patterns, flags=None):
         """Removes ``patterns`` from the given ``string``.
 
         This keyword is otherwise identical to `Remove String`, but
@@ -405,12 +515,18 @@ class String(object):
         used if there is a need to remove only a certain number of
         occurrences.
 
-        New in Robot Framework 2.8.2.
+        Possible flags altering how the expression is parsed (e.g. ``re.IGNORECASE``,
+        ``re.MULTILINE``) can be given using the ``flags`` argument (e.g.
+        ``flags=IGNORECASE | MULTILINE``) or embedded to the pattern (e.g.
+        ``(?im)pattern``).
+
+        The ``flags`` argument is new in Robot Framework 6.0.
         """
         for pattern in patterns:
-            string = self.replace_string_using_regexp(string, pattern, '')
+            string = self.replace_string_using_regexp(string, pattern, '', flags=flags)
         return string
 
+    @keyword(types=None)
     def split_string(self, string, separator=None, max_split=-1):
         """Splits the ``string`` using ``separator`` as a delimiter string.
 
@@ -436,6 +552,7 @@ class String(object):
         max_split = self._convert_to_integer(max_split, 'max_split')
         return string.split(separator, max_split)
 
+    @keyword(types=None)
     def split_string_from_right(self, string, separator=None, max_split=-1):
         """Splits the ``string`` using ``separator`` starting from right.
 
@@ -456,8 +573,6 @@ class String(object):
 
         Example:
         | @{characters} = | Split String To Characters | ${string} |
-
-        New in Robot Framework 2.7.
         """
         return list(string)
 
@@ -484,6 +599,10 @@ class String(object):
     def generate_random_string(self, length=8, chars='[LETTERS][NUMBERS]'):
         """Generates a string with a desired ``length`` from the given ``chars``.
 
+        ``length`` can be given as a number, a string representation of a number,
+        or as a range of numbers, such as ``5-10``. When a range of values is given
+        the range will be selected by random within the range.
+
         The population sequence ``chars`` contains the characters to use
         when generating the random string. It can contain any
         characters, and it is possible to use special markers
@@ -500,10 +619,18 @@ class String(object):
         | ${low} = | Generate Random String | 12 | [LOWER]         |
         | ${bin} = | Generate Random String | 8  | 01              |
         | ${hex} = | Generate Random String | 4  | [NUMBERS]abcdef |
+        | ${rnd} = | Generate Random String | 5-10 | # Generates a string 5 to 10 characters long |
+
+        Giving ``length`` as a range of values is new in Robot Framework 5.0.
         """
         if length == '':
             length = 8
-        length = self._convert_to_integer(length, 'length')
+        if isinstance(length, str) and re.match(r'^\d+-\d+$', length):
+            min_length, max_length = length.split('-')
+            length = randint(self._convert_to_integer(min_length, "length"),
+                             self._convert_to_integer(max_length, "length"))
+        else:
+            length = self._convert_to_integer(length, 'length')
         for name, value in [('[LOWER]', ascii_lowercase),
                             ('[UPPER]', ascii_uppercase),
                             ('[LETTERS]', ascii_lowercase + ascii_uppercase),
@@ -530,6 +657,7 @@ class String(object):
         end = self._convert_to_index(end, 'end')
         return string[start:end]
 
+    @keyword(types=None)
     def strip_string(self, string, mode='both', characters=None):
         """Remove leading and/or trailing whitespaces from the given string.
 
@@ -550,8 +678,6 @@ class String(object):
         | Should Be Equal | ${stripped} | Hello${SPACE} | |
         | ${stripped}=  | Strip String | aabaHelloeee | characters=abe |
         | Should Be Equal | ${stripped} | Hello | |
-
-        New in Robot Framework 3.0.
         """
         try:
             method = {'BOTH': string.strip,
@@ -565,21 +691,15 @@ class String(object):
     def should_be_string(self, item, msg=None):
         """Fails if the given ``item`` is not a string.
 
-        This keyword passes regardless is the ``item`` is a Unicode string or
-        a byte string. Use `Should Be Unicode String` or `Should Be Byte
-        String` if you want to restrict the string type.
-
-        The default error message can be overridden with the optional
-        ``msg`` argument.
+        The default error message can be overridden with the optional ``msg`` argument.
         """
         if not is_string(item):
-            self._fail(msg, "'%s' is not a string.", item)
+            self._fail(msg, "'%s' is %s, not a string.", item, type_name(item))
 
     def should_not_be_string(self, item, msg=None):
         """Fails if the given ``item`` is a string.
 
-        The default error message can be overridden with the optional
-        ``msg`` argument.
+        The default error message can be overridden with the optional ``msg`` argument.
         """
         if is_string(item):
             self._fail(msg, "'%s' is a string.", item)
@@ -587,35 +707,24 @@ class String(object):
     def should_be_unicode_string(self, item, msg=None):
         """Fails if the given ``item`` is not a Unicode string.
 
-        Use `Should Be Byte String` if you want to verify the ``item`` is a
-        byte string, or `Should Be String` if both Unicode and byte strings
-        are fine.
-
-        The default error message can be overridden with the optional
-        ``msg`` argument.
-
-        New in Robot Framework 2.7.7.
+        On Python 3 this keyword behaves exactly the same way `Should Be String`.
+        That keyword should be used instead and this keyword will be deprecated.
         """
-        if not is_unicode(item):
+        if not is_string(item):
             self._fail(msg, "'%s' is not a Unicode string.", item)
 
     def should_be_byte_string(self, item, msg=None):
         """Fails if the given ``item`` is not a byte string.
 
-        Use `Should Be Unicode String` if you want to verify the ``item`` is a
-        Unicode string, or `Should Be String` if both Unicode and byte strings
-        are fine.
+        Use `Should Be String` if you want to verify the ``item`` is a string.
 
-        The default error message can be overridden with the optional
-        ``msg`` argument.
-
-        New in Robot Framework 2.7.7.
+        The default error message can be overridden with the optional ``msg`` argument.
         """
         if not is_bytes(item):
             self._fail(msg, "'%s' is not a byte string.", item)
 
-    def should_be_lowercase(self, string, msg=None):
-        """Fails if the given ``string`` is not in lowercase.
+    def should_be_lower_case(self, string, msg=None):
+        """Fails if the given ``string`` is not in lower case.
 
         For example, ``'string'`` and ``'with specials!'`` would pass, and
         ``'String'``, ``''`` and ``' '`` would fail.
@@ -623,13 +732,13 @@ class String(object):
         The default error message can be overridden with the optional
         ``msg`` argument.
 
-        See also `Should Be Uppercase` and `Should Be Titlecase`.
+        See also `Should Be Upper Case` and `Should Be Title Case`.
         """
         if not string.islower():
-            self._fail(msg, "'%s' is not lowercase.", string)
+            self._fail(msg, "'%s' is not lower case.", string)
 
-    def should_be_uppercase(self, string, msg=None):
-        """Fails if the given ``string`` is not in uppercase.
+    def should_be_upper_case(self, string, msg=None):
+        """Fails if the given ``string`` is not in upper case.
 
         For example, ``'STRING'`` and ``'WITH SPECIALS!'`` would pass, and
         ``'String'``, ``''`` and ``' '`` would fail.
@@ -637,28 +746,43 @@ class String(object):
         The default error message can be overridden with the optional
         ``msg`` argument.
 
-        See also `Should Be Titlecase` and `Should Be Lowercase`.
+        See also `Should Be Title Case` and `Should Be Lower Case`.
         """
         if not string.isupper():
-            self._fail(msg, "'%s' is not uppercase.", string)
+            self._fail(msg, "'%s' is not upper case.", string)
 
-    def should_be_titlecase(self, string, msg=None):
+    @keyword(types=None)
+    def should_be_title_case(self, string, msg=None, exclude=None):
         """Fails if given ``string`` is not title.
 
-        ``string`` is a titlecased string if there is at least one
-        character in it, uppercase characters only follow uncased
-        characters and lowercase characters only cased ones.
+        ``string`` is a title cased string if there is at least one upper case
+        letter in each word.
 
-        For example, ``'This Is Title'`` would pass, and ``'Word In UPPER'``,
-        ``'Word In lower'``, ``''`` and ``' '`` would fail.
+        For example, ``'This Is Title'`` and ``'OK, Give Me My iPhone'``
+        would pass. ``'all words lower'`` and ``'Word In lower'`` would fail.
+
+        This logic changed in Robot Framework 4.0 to be compatible with
+        `Convert to Title Case`. See `Convert to Title Case` for title case
+        algorithm and reasoning.
 
         The default error message can be overridden with the optional
         ``msg`` argument.
 
-        See also `Should Be Uppercase` and `Should Be Lowercase`.
+        Words can be explicitly excluded with the optional ``exclude`` argument.
+
+        Explicitly excluded words can be given as a list or as a string with
+        words separated by a comma and an optional space. Excluded words are
+        actually considered to be regular expression patterns, so it is
+        possible to use something like "example[.!?]?" to match the word
+        "example" on it own and also if followed by ".", "!" or "?".
+        See `BuiltIn.Should Match Regexp` for more information about Python
+        regular expression syntax in general and how to use it in Robot
+        Framework data in particular.
+
+        See also `Should Be Upper Case` and `Should Be Lower Case`.
         """
-        if not string.istitle():
-            self._fail(msg, "'%s' is not titlecase.", string)
+        if string != self.convert_to_title_case(string, exclude):
+            self._fail(msg, "'%s' is not title case.", string)
 
     def _convert_to_index(self, value, name):
         if value == '':
@@ -676,5 +800,5 @@ class String(object):
 
     def _fail(self, message, default_template, *items):
         if not message:
-            message = default_template % tuple(unic(item) for item in items)
+            message = default_template % tuple(safe_str(item) for item in items)
         raise AssertionError(message)
