@@ -1,4 +1,5 @@
-#  Copyright 2008-2015 Nokia Solutions and Networks
+#  Copyright 2008-2015 Nokia Networks
+#  Copyright 2016-     Robot Framework Foundation
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,21 +13,20 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 
-from robot.utils import is_string, NormalizedDict, py2to3, unic
+from robot.utils import is_string, NormalizedDict
 
 
-@py2to3
 class Metadata(NormalizedDict):
 
     def __init__(self, initial=None):
-        NormalizedDict.__init__(self, initial, ignore='_')
+        super().__init__(initial, ignore='_')
 
     def __setitem__(self, key, value):
         if not is_string(key):
-            key = unic(key)
+            key = str(key)
         if not is_string(value):
-            value = unic(value)
-        NormalizedDict.__setitem__(self, key, value)
+            value = str(value)
+        super().__setitem__(key, value)
 
-    def __unicode__(self):
-        return u'{%s}' % ', '.join('%s: %s' % (k, self[k]) for k in self)
+    def __str__(self):
+        return '{%s}' % ', '.join('%s: %s' % (k, self[k]) for k in self)
