@@ -13,11 +13,10 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 
-from robot.utils import MultiMatcher, py2to3
+from robot.utils import MultiMatcher
 
 
-@py2to3
-class _NamePatterns(object):
+class NamePatterns:
 
     def __init__(self, patterns=None):
         self._matcher = MultiMatcher(patterns, ignore='_')
@@ -31,14 +30,14 @@ class _NamePatterns(object):
     def _match_longname(self, name):
         raise NotImplementedError
 
-    def __nonzero__(self):
+    def __bool__(self):
         return bool(self._matcher)
 
     def __iter__(self):
         return iter(self._matcher)
 
 
-class SuiteNamePatterns(_NamePatterns):
+class SuiteNamePatterns(NamePatterns):
 
     def _match_longname(self, name):
         while '.' in name:
@@ -48,7 +47,7 @@ class SuiteNamePatterns(_NamePatterns):
         return False
 
 
-class TestNamePatterns(_NamePatterns):
+class TestNamePatterns(NamePatterns):
 
     def _match_longname(self, name):
         return self._match(name)
