@@ -14,14 +14,13 @@
 #  limitations under the License.
 
 from robot.errors import TimeoutError
-from robot.utils import get_error_details, py2to3
+from robot.utils import get_error_details
 
 from .listenerarguments import ListenerArguments
 from .logger import LOGGER
 
 
-@py2to3
-class ListenerMethods(object):
+class ListenerMethods:
 
     def __init__(self, method_name, listeners):
         self._methods = []
@@ -41,11 +40,11 @@ class ListenerMethods(object):
             for method in self._methods:
                 method(args.get_arguments(method.version))
 
-    def __nonzero__(self):
+    def __bool__(self):
         return bool(self._methods)
 
 
-class LibraryListenerMethods(object):
+class LibraryListenerMethods:
 
     def __init__(self, method_name):
         self._method_stack = []
@@ -85,7 +84,7 @@ class LibraryListenerMethods(object):
         return methods
 
 
-class ListenerMethod(object):
+class ListenerMethod:
     # Flag to avoid recursive listener calls.
     called = False
 
