@@ -15,11 +15,12 @@
 
 from robot.errors import DataError
 from robot.model import Message as BaseMessage
-from robot.utils import get_timestamp, is_unicode, unic
+from robot.utils import get_timestamp, is_string, safe_str
 
 
 LEVELS = {
-  'NONE'  : 6,
+  'NONE'  : 7,
+  'SKIP'  : 6,
   'FAIL'  : 5,
   'ERROR' : 4,
   'WARN'  : 3,
@@ -29,7 +30,7 @@ LEVELS = {
 }
 
 
-class AbstractLogger(object):
+class AbstractLogger:
 
     def __init__(self, level='TRACE'):
         self._is_logged = IsLogged(level)
@@ -56,6 +57,13 @@ class AbstractLogger(object):
             msg = msg[6:].lstrip()
         self.write(msg, 'FAIL', html)
 
+    def skip(self, msg):
+        html = False
+        if msg.startswith("*HTML*"):
+            html = True
+            msg = msg[6:].lstrip()
+        self.write(msg, 'SKIP', html)
+
     def error(self, msg):
         self.write(msg, 'ERROR')
 
@@ -73,13 +81,13 @@ class Message(BaseMessage):
         message = self._normalize_message(message)
         level, html = self._get_level_and_html(level, html)
         timestamp = timestamp or get_timestamp()
-        BaseMessage.__init__(self, message, level, html, timestamp)
+        super().__init__(message, level, html, timestamp)
 
     def _normalize_message(self, msg):
         if callable(msg):
             return msg
-        if not is_unicode(msg):
-            msg = unic(msg)
+        if not is_string(msg):
+            msg = safe_str(msg)
         if '\r\n' in msg:
             msg = msg.replace('\r\n', '\n')
         return msg
@@ -106,17 +114,17 @@ class Message(BaseMessage):
             self._message = self._message()
 
 
-class IsLogged(object):
+class IsLogged:
 
     def __init__(self, level):
-        self._str_level = level
+        self.level = level.upper()
         self._int_level = self._level_to_int(level)
 
     def __call__(self, level):
         return self._level_to_int(level) >= self._int_level
 
     def set_level(self, level):
-        old = self._str_level.upper()
+        old = self.level
         self.__init__(level)
         return old
 
@@ -127,14 +135,21 @@ class IsLogged(object):
             raise DataError("Invalid log level '%s'." % level)
 
 
-class AbstractLoggerProxy(object):
+class AbstractLoggerProxy:
     _methods = None
     _no_method = lambda *args: None
 
     def __init__(self, logger, method_names=None, prefix=None):
         self.logger = logger
         for name in method_names or self._methods:
-            setattr(self, name, self._get_method(logger, name, prefix))
+            # Allow extending classes to implement some of the messages themselves.
+            if hasattr(self, name):
+                if hasattr(logger, name):
+                    continue
+                target = logger
+            else:
+                target = self
+            setattr(target, name, self._get_method(logger, name, prefix))
 
     def _get_method(self, logger, name, prefix):
         for method_name in self._get_method_names(name, prefix):
