@@ -28,10 +28,11 @@ def KeywordRemover(how):
         return {'ALL': AllKeywordsRemover,
                 'PASSED': PassedKeywordRemover,
                 'FOR': ForLoopItemsRemover,
+                'WHILE': WhileLoopItemsRemover,
                 'WUKS': WaitUntilKeywordSucceedsRemover}[upper]()
     except KeyError:
-        raise DataError("Expected 'ALL', 'PASSED', 'NAME:<pattern>', 'TAG:<pattern>', "
-                        "'FOR', or 'WUKS' but got '%s'." % how)
+        raise DataError(f"Expected 'ALL', 'PASSED', 'NAME:<pattern>', 'TAG:<pattern>', "
+                        f"'FOR' or 'WUKS', got '{how}'.")
 
 
 class _KeywordRemover(SuiteVisitor):
@@ -40,10 +41,9 @@ class _KeywordRemover(SuiteVisitor):
     def __init__(self):
         self._removal_message = RemovalMessage(self._message)
 
-    def _clear_content(self, kw):
-        kw.keywords = []
-        kw.messages = []
-        self._removal_message.set(kw)
+    def _clear_content(self, item):
+        item.body.clear()
+        self._removal_message.set(item)
 
     def _failed_or_warning_or_error(self, item):
         return not item.passed or self._warning_or_error(item)
@@ -59,18 +59,24 @@ class AllKeywordsRemover(_KeywordRemover):
     def visit_keyword(self, keyword):
         self._clear_content(keyword)
 
+    def visit_for(self, for_):
+        self._clear_content(for_)
+
+    def visit_if_branch(self, branch):
+        self._clear_content(branch)
+
 
 class PassedKeywordRemover(_KeywordRemover):
 
     def start_suite(self, suite):
-        if not suite.statistics.all.failed:
-            for keyword in suite.keywords:
+        if not suite.statistics.failed:
+            for keyword in suite.setup, suite.teardown:
                 if not self._warning_or_error(keyword):
                     self._clear_content(keyword)
 
     def visit_test(self, test):
         if not self._failed_or_warning_or_error(test):
-            for keyword in test.keywords:
+            for keyword in test.body:
                 self._clear_content(keyword)
 
     def visit_keyword(self, keyword):
@@ -99,36 +105,49 @@ class ByTagKeywordRemover(_KeywordRemover):
             self._clear_content(kw)
 
 
-class ForLoopItemsRemover(_KeywordRemover):
+class _LoopItemsRemover(_KeywordRemover):
     _message = '%d passing step%s removed using --RemoveKeywords option.'
 
-    def start_keyword(self, kw):
-        if kw.type == kw.FOR_LOOP_TYPE:
-            before = len(kw.keywords)
-            kw.keywords = self._remove_keywords(kw.keywords)
-            self._removal_message.set_if_removed(kw, before)
+    def _remove_from_loop(self, loop):
+        before = len(loop.body)
+        self._remove_keywords(loop.body)
+        self._removal_message.set_if_removed(loop, before)
 
-    def _remove_keywords(self, keywords):
-        return [kw for kw in keywords
-                if self._failed_or_warning_or_error(kw) or kw is keywords[-1]]
+    def _remove_keywords(self, body):
+        iterations = body.filter(messages=False)
+        for it in iterations[:-1]:
+            if not self._failed_or_warning_or_error(it):
+                body.remove(it)
+
+
+class ForLoopItemsRemover(_LoopItemsRemover):
+
+    def start_for(self, for_):
+        self._remove_from_loop(for_)
+
+
+class WhileLoopItemsRemover(_LoopItemsRemover):
+
+    def start_while(self, while_):
+        self._remove_from_loop(while_)
 
 
 class WaitUntilKeywordSucceedsRemover(_KeywordRemover):
     _message = '%d failing step%s removed using --RemoveKeywords option.'
 
     def start_keyword(self, kw):
-        if kw.name == 'BuiltIn.Wait Until Keyword Succeeds' and kw.keywords:
-            before = len(kw.keywords)
-            kw.keywords = self._remove_keywords(list(kw.keywords))
+        if kw.libname == 'BuiltIn' and kw.kwname == 'Wait Until Keyword Succeeds':
+            before = len(kw.body)
+            self._remove_keywords(kw.body)
             self._removal_message.set_if_removed(kw, before)
 
-    def _remove_keywords(self, keywords):
-        include_from_end = 2 if keywords[-1].passed else 1
-        return self._kws_with_warnings(keywords[:-include_from_end]) \
-            + keywords[-include_from_end:]
-
-    def _kws_with_warnings(self, keywords):
-        return [kw for kw in keywords if self._warning_or_error(kw)]
+    def _remove_keywords(self, body):
+        keywords = body.filter(messages=False)
+        if keywords:
+            include_from_end = 2 if keywords[-1].passed else 1
+            for kw in keywords[:-include_from_end]:
+                if not self._warning_or_error(kw):
+                    body.remove(kw)
 
 
 class WarningAndErrorFinder(SuiteVisitor):
@@ -150,13 +169,13 @@ class WarningAndErrorFinder(SuiteVisitor):
             self.found = True
 
 
-class RemovalMessage(object):
+class RemovalMessage:
 
     def __init__(self, message):
         self._message = message
 
     def set_if_removed(self, kw, len_before):
-        removed = len_before - len(kw.keywords)
+        removed = len_before - len(kw.body)
         if removed:
             self.set(kw, self._message % (removed, plural_or_not(removed)))
 
