@@ -41,33 +41,30 @@ from robot.conf import RobotSettings
 from robot.model import ModelModifier
 from robot.output import LOGGER, pyloggingconf
 from robot.reporting import ResultWriter
-from robot.running import TestSuiteBuilder
-from robot.utils import Application, unic, text
+from robot.running.builder import TestSuiteBuilder
+from robot.utils import Application, text
 
 
 USAGE = """Robot Framework -- A generic automation framework
 
 Version:  <VERSION>
 
-Usage:  robot [options] data_sources
-   or:  python -m robot [options] data_sources
-   or:  python path/to/robot [options] data_sources
-   or:  java -jar robotframework.jar [options] data_sources
+Usage:  robot [options] paths
+   or:  python -m robot [options] paths
+   or:  python path/to/robot [options] paths
 
 Robot Framework is a generic open source automation framework for acceptance
 testing, acceptance test-driven development (ATDD) and robotic process
 automation (RPA). It has simple, easy-to-use syntax that utilizes the
 keyword-driven automation approach. Keywords adding new capabilities are
-implemented in libraries using either Python or Java. New higher level
+implemented in libraries using Python. New higher level
 keywords can also be created using Robot Framework's own syntax.
 
 The easiest way to execute Robot Framework is using the `robot` command created
 as part of the normal installation. Alternatively it is possible to execute
-the `robot` module directly using `python -m robot`, where `python` can be
-replaced with any supported Python interpreter such as `jython`, `ipy` or
-`python3`. Yet another alternative is running the `robot` directory like
-`python path/to/robot`. Finally, there is a standalone JAR distribution
-available.
+the `robot` module directly like `python -m robot`, where `python` can be
+replaced with any supported Python interpreter. Yet another alternative
+is running the `robot` directory like `python path/to/robot`.
 
 Tests (or tasks in RPA terminology) are created in files typically having the
 `*.robot` extension. Files automatically create test (or task) suites and
@@ -88,59 +85,69 @@ see http://robotframework.org/.
 Options
 =======
 
-    --rpa                 Turn on generic automation mode. Mainly affects
+    --rpa                 Turn on the generic automation mode. Mainly affects
                           terminology so that "test" is replaced with "task"
                           in logs and reports. By default the mode is got
-                          from test/task header in data files. New in RF 3.1.
+                          from test/task header in data files.
+    --language lang *     Activate localization. `lang` can be a name or a code
+                          of a built-in language, or a path or a module name of
+                          a custom language file.
  -F --extension value     Parse only files with this extension when executing
                           a directory. Has no effect when running individual
                           files or when using resource files. If more than one
                           extension is needed, separate them with a colon.
-                          Examples: `--extension robot`, `-F robot:txt`
-                          New in RF 3.0.1.
- -N --name name           Set the name of the top level test suite. Default
-                          name is created from the name of the executed data
-                          source.
- -D --doc documentation   Set the documentation of the top level test suite.
-                          Simple formatting is supported (e.g. *bold*). If
-                          the documentation contains spaces, it must be quoted.
-                          Example: --doc "Very *good* example"
+                          Examples: `--extension txt`, `--extension robot:txt`
+                          Only `*.robot` files are parsed by default.
+ -N --name name           Set the name of the top level suite. By default the
+                          name is created based on the executed file or
+                          directory.
+ -D --doc documentation   Set the documentation of the top level suite.
+                          Simple formatting is supported (e.g. *bold*). If the
+                          documentation contains spaces, it must be quoted.
+                          If the value is path to an existing file, actual
+                          documentation is read from that file.
+                          Examples: --doc "Very *good* example"
+                                    --doc doc_from_file.txt
  -M --metadata name:value *  Set metadata of the top level suite. Value can
-                          contain formatting similarly as --doc.
-                          Example: --metadata version:1.2
- -G --settag tag *        Sets given tag(s) to all executed test cases.
- -t --test name *         Select test cases to run by name or long name. Name
-                          is case and space insensitive and it can also be a
-                          simple pattern where `*` matches anything and `?`
-                          matches any char.
+                          contain formatting and be read from a file similarly
+                          as --doc. Example: --metadata Version:1.2
+ -G --settag tag *        Sets given tag(s) to all executed tests.
+ -t --test name *         Select tests by name or by long name containing also
+                          parent suite name like `Parent.Test`. Name is case
+                          and space insensitive and it can also be a simple
+                          pattern where `*` matches anything, `?` matches any
+                          single character, and `[chars]` matches one character
+                          in brackets.
     --task name *         Alias to --test. Especially applicable with --rpa.
- -s --suite name *        Select test suites to run by name. When this option
-                          is used with --test, --include or --exclude, only
-                          test cases in matching suites and also matching other
-                          filtering criteria are selected. Name can be a simple
-                          pattern similarly as with --test and it can contain
-                          parent name separated with a dot. For example
-                          `-s X.Y` selects suite `Y` only if its parent is `X`.
- -i --include tag *       Select test cases to run by tag. Similarly as name
-                          with --test, tag is case and space insensitive and it
-                          is possible to use patterns with `*` and `?` as
-                          wildcards. Tags and patterns can also be combined
-                          together with `AND`, `OR`, and `NOT` operators.
+ -s --suite name *        Select suites by name. When this option is used with
+                          --test, --include or --exclude, only tests in
+                          matching suites and also matching other filtering
+                          criteria are selected. Name can be a simple pattern
+                          similarly as with --test and it can contain parent
+                          name separated with a dot. For example, `-s X.Y`
+                          selects suite `Y` only if its parent is `X`.
+ -i --include tag *       Select tests by tag. Similarly as name with --test,
+                          tag is case and space insensitive and it is possible
+                          to use patterns with `*`, `?` and `[]` as wildcards.
+                          Tags and patterns can also be combined together with
+                          `AND`, `OR`, and `NOT` operators.
                           Examples: --include foo --include bar*
                                     --include fooANDbar*
  -e --exclude tag *       Select test cases not to run by tag. These tests are
                           not run even if included with --include. Tags are
-                          matched using the rules explained with --include.
+                          matched using same rules as with --include.
  -R --rerunfailed output  Select failed tests from an earlier output file to be
                           re-executed. Equivalent to selecting same tests
-                          individually using --test option.
- -S --rerunfailedsuites output  Select failed suite from an earlier output file
-                          to be re-executed. New in RF 3.0.1.
- -c --critical tag *      Tests having given tag are considered critical. If no
-                          critical tags are set, all tags are critical. Tags
-                          can be given as a pattern like with --include.
- -n --noncritical tag *   Tests with given tag are not critical even if they
-                          have a tag set with --critical. Tag can be a pattern.
+                          individually using --test.
+ -S --rerunfailedsuites output  Select failed suites from an earlier output
+                          file to be re-executed.
+    --runemptysuite       Executes suite even if it contains no tests. Useful
+                          e.g. with --include/--exclude when it is not an error
+                          that no test matches the condition.
+    --skip tag *          Tests having given tag will be skipped. Tag can be
+                          a pattern.
+    --skiponfailure tag *  Tests having given tag will be skipped if they fail.
+                          Tag can be a pattern
  -v --variable name:value *  Set variables in the test data. Only scalar
                           variables with string value are supported and name is
                           given without `${}`. See --variablefile for a more
@@ -163,8 +170,7 @@ Options
                           path. Other output files are created based on XML
                           output files after the test execution and XML outputs
                           can also be further processed with Rebot tool. Can be
-                          disabled by giving a special value `NONE`. In this
-                          case, also log and report are automatically disabled.
+                          disabled by giving a special value `NONE`.
                           Default: output.xml
  -l --log file            HTML log file. Can be disabled by giving a special
                           value `NONE`. Default: log.html
@@ -173,7 +179,6 @@ Options
                           similarly as --log. Default: report.html
  -x --xunit file          xUnit compatible result file. Not created unless this
                           option is specified.
-    --xunitskipnoncritical  Mark non-critical tests on xUnit output as skipped.
  -b --debugfile file      Debug file written during execution. Not created
                           unless this option is specified.
  -T --timestampoutputs    When this option is used, timestamp in a format
@@ -182,20 +187,25 @@ Options
                           example `-T -o output.xml -r report.html -l none`
                           creates files like `output-20070503-154410.xml` and
                           `report-20070503-154410.html`.
-    --splitlog            Split log file into smaller pieces that open in
-                          browser transparently.
-    --logtitle title      Title for the generated test log. The default title
-                          is `<Name Of The Suite> Test Log`.
-    --reporttitle title   Title for the generated test report. The default
-                          title is `<Name Of The Suite> Test Report`.
+    --splitlog            Split the log file into smaller pieces that open in
+                          browsers transparently.
+    --logtitle title      Title for the generated log file. The default title
+                          is `<SuiteName> Log`.
+    --reporttitle title   Title for the generated report file. The default
+                          title is `<SuiteName> Report`.
     --reportbackground colors  Background colors to use in the report file.
-                          Either `all_passed:critical_passed:failed` or
-                          `passed:failed`. Both color names and codes work.
-                          Examples: --reportbackground green:yellow:red
+                          Given in format `passed:failed:skipped` where the
+                          `:skipped` part can be omitted. Both color names and
+                          codes work.
+                          Examples: --reportbackground green:red:yellow
                                     --reportbackground #00E:#E00
     --maxerrorlines lines  Maximum number of error message lines to show in
                           report when tests fail. Default is 40, minimum is 10
                           and `NONE` can be used to show the full message.
+    --maxassignlength characters  Maximum number of characters to show in log
+                          when variables are assigned. Zero or negative values
+                          can be used to avoid showing assigned values at all.
+                          Default is 200.
  -L --loglevel level      Threshold level for logging. Available levels: TRACE,
                           DEBUG, INFO (default), WARN, NONE (no logging). Use
                           syntax `LOGLEVEL:DEFAULT` to define the default
@@ -206,45 +216,47 @@ Options
                           in log and report. By default all suite levels are
                           shown. Example:  --suitestatlevel 3
     --tagstatinclude tag *  Include only matching tags in `Statistics by Tag`
-                          and `Test Details` in log and report. By default all
-                          tags set in test cases are shown. Given `tag` can
-                          also be a simple pattern (see e.g. --test).
-    --tagstatexclude tag *  Exclude matching tags from `Statistics by Tag` and
-                          `Test Details`. This option can be used with
-                          --tagstatinclude similarly as --exclude is used with
-                          --include.
+                          in log and report. By default all tags are shown.
+                          Given tag can be a pattern like with --include.
+    --tagstatexclude tag *  Exclude matching tags from `Statistics by Tag`.
+                          This option can be used with --tagstatinclude
+                          similarly as --exclude is used with --include.
     --tagstatcombine tags:name *  Create combined statistics based on tags.
-                          These statistics are added into `Statistics by Tag`
-                          and matching tests into `Test Details`. If optional
-                          `name` is not given, name of the combined tag is got
-                          from the specified tags. Tags are combined using the
-                          rules explained in --include.
+                          These statistics are added into `Statistics by Tag`.
+                          If the optional `name` is not given, name of the
+                          combined tag is got from the specified tags. Tags are
+                          matched using the same rules as with --include.
                           Examples: --tagstatcombine requirement-*
                                     --tagstatcombine tag1ANDtag2:My_name
-    --tagdoc pattern:doc *  Add documentation to tags matching given pattern.
-                          Documentation is shown in `Test Details` and also as
-                          a tooltip in `Statistics by Tag`. Pattern can contain
-                          characters `*` (matches anything) and `?` (matches
-                          any char). Documentation can contain formatting
-                          similarly as with --doc option.
+    --tagdoc pattern:doc *  Add documentation to tags matching the given
+                          pattern. Documentation is shown in `Test Details` and
+                          also as a tooltip in `Statistics by Tag`. Pattern can
+                          use `*`, `?` and `[]` as wildcards like --test.
+                          Documentation can contain formatting like --doc.
                           Examples: --tagdoc mytag:Example
                                     --tagdoc "owner-*:Original author"
     --tagstatlink pattern:link:title *  Add external links into `Statistics by
-                          Tag`. Pattern can contain characters `*` (matches
-                          anything) and `?` (matches any char). Characters
-                          matching to wildcard expressions can be used in link
-                          and title with syntax %N, where N is index of the
-                          match (starting from 1).
+                          Tag`. Pattern can use `*`, `?` and `[]` as wildcards
+                          like --test. Characters matching to `*` and `?`
+                          wildcards can be used in link and title with syntax
+                          %N, where N is index of the match (starting from 1).
                           Examples: --tagstatlink mytag:http://my.domain:Title
                           --tagstatlink "bug-*:http://url/id=%1:Issue Tracker"
+    --expandkeywords name:<pattern>|tag:<pattern> *
+                          Matching keywords will be automatically expanded in
+                          the log file. Matching against keyword name or tags
+                          work using same rules as with --removekeywords.
+                          Examples: --expandkeywords name:BuiltIn.Log
+                                    --expandkeywords tag:expand
     --removekeywords all|passed|for|wuks|name:<pattern>|tag:<pattern> *
                           Remove keyword data from the generated log file.
                           Keywords containing warnings are not removed except
-                          in `all` mode.
+                          in the `all` mode.
                           all:     remove data from all keywords
                           passed:  remove data only from keywords in passed
                                    test cases and suites
                           for:     remove passed iterations from for loops
+                          while:   remove passed iterations from while loops
                           wuks:    remove all but the last failing keyword
                                    inside `BuiltIn.Wait Until Keyword Succeeds`
                           name:<pattern>:  remove data from keywords that match
@@ -252,24 +264,25 @@ Options
                                    against the full name of the keyword (e.g.
                                    'MyLib.Keyword', 'resource.Second Keyword'),
                                    is case, space, and underscore insensitive,
-                                   and may contain `*` and `?` as wildcards.
+                                   and may contain `*`, `?` and `[]` wildcards.
                                    Examples: --removekeywords name:Lib.HugeKw
                                              --removekeywords name:myresource.*
                           tag:<pattern>:  remove data from keywords that match
                                    the given pattern. Tags are case and space
-                                   insensitive and it is possible to use
-                                   patterns with `*` and `?` as wildcards.
-                                   Tags and patterns can also be combined
-                                   together with `AND`, `OR`, and `NOT`
-                                   operators.
+                                   insensitive and patterns can contain `*`,
+                                   `?` and `[]` wildcards. Tags and patterns
+                                   can also be combined together with `AND`,
+                                   `OR`, and `NOT` operators.
                                    Examples: --removekeywords foo
                                              --removekeywords fooANDbar*
-    --flattenkeywords for|foritem|name:<pattern>|tag:<pattern> *
+    --flattenkeywords for|while|iteration|name:<pattern>|tag:<pattern> *
                           Flattens matching keywords in the generated log file.
                           Matching keywords get all log messages from their
                           child keywords and children are discarded otherwise.
-                          for:     flatten for loops fully
-                          foritem: flatten individual for loop iterations
+                          for:     flatten FOR loops fully
+                          while:   flatten WHILE loops fully
+                          iteration: flatten FOR/WHILE loop iterations
+                          foritem: deprecated alias for `iteration`
                           name:<pattern>:  flatten matched keywords using same
                                    matching rules as with
                                    `--removekeywords name:<pattern>`
@@ -277,21 +290,16 @@ Options
                                    matching rules as with
                                    `--removekeywords tag:<pattern>`
     --listener class *    A class for monitoring test execution. Gets
-                          notifications e.g. when a test case starts and ends.
+                          notifications e.g. when tests start and end.
                           Arguments to the listener class can be given after
-                          the name using colon or semicolon as a separator.
+                          the name using a colon or a semicolon as a separator.
                           Examples: --listener MyListenerClass
                                     --listener path/to/Listener.py:arg1:arg2
-    --warnonskippedfiles  Deprecated. Nowadays all skipped files are reported.
     --nostatusrc          Sets the return code to zero regardless of failures
                           in test cases. Error codes are returned normally.
-    --runemptysuite       Executes tests also if the top level test suite is
-                          empty. Useful e.g. with --include/--exclude when it
-                          is not an error that no test matches the condition.
     --dryrun              Verifies test data and runs tests so that library
                           keywords are not executed.
- -X --exitonfailure       Stops test execution if any critical test fails.
-                          Short option -X is new in RF 3.0.1.
+ -X --exitonfailure       Stops test execution if any test fails.
     --exitonerror         Stops test execution if any error occurs when parsing
                           test data, importing libraries, and so on.
     --skipteardownonexit  Causes teardowns to be skipped if test execution is
@@ -305,51 +313,47 @@ Options
                           The seed must be an integer.
                           Examples: --randomize all
                                     --randomize tests:1234
-    --prerunmodifier class *  Class to programmatically modify the test suite
+    --prerunmodifier class *  Class to programmatically modify the suite
                           structure before execution.
     --prerebotmodifier class *  Class to programmatically modify the result
                           model before creating reports and logs.
     --console type        How to report execution on the console.
                           verbose:  report every suite and test (default)
-                          dotted:   only show `.` for passed test, `f` for
-                                    failed non-critical tests, and `F` for
-                                    failed critical tests
+                          dotted:   only show `.` for passed test, `s` for
+                                    skipped tests, and `F` for failed tests
                           quiet:    no output except for errors and warnings
                           none:     no output whatsoever
  -. --dotted              Shortcut for `--console dotted`.
     --quiet               Shortcut for `--console quiet`.
- -W --consolewidth chars  Width of the monitor output. Default is 78.
+ -W --consolewidth chars  Width of the console output. Default is 78.
  -C --consolecolors auto|on|ansi|off  Use colors on console output or not.
                           auto: use colors when output not redirected (default)
                           on:   always use colors
                           ansi: like `on` but use ANSI colors also on Windows
                           off:  disable colors altogether
-                          Note that colors do not work with Jython on Windows.
  -K --consolemarkers auto|on|off  Show markers on the console when top level
                           keywords in a test case end. Values have same
                           semantics as with --consolecolors.
- -P --pythonpath path *   Additional locations (directories, ZIPs, JARs) where
-                          to search test libraries and other extensions when
-                          they are imported. Multiple paths can be given by
-                          separating them with a colon (`:`) or by using this
-                          option several times. Given path can also be a glob
-                          pattern matching multiple paths.
-                          Examples:
-                          --pythonpath libs/ --pythonpath resources/*.jar
-                          --pythonpath /opt/testlibs:mylibs.zip:yourlibs
- -E --escape what:with *  Deprecated. Use console escape mechanism instead.
+ -P --pythonpath path *   Additional locations (directories, ZIPs) where to
+                          search libraries and other extensions when they are
+                          imported. Multiple paths can be given by separating
+                          them with a colon (`:`) or by using this option
+                          several times. Given path can also be a glob pattern
+                          matching multiple paths.
+                          Examples: --pythonpath libs/
+                                    --pythonpath /opt/libs:libraries.zip
  -A --argumentfile path *  Text file to read more arguments from. Use special
                           path `STDIN` to read contents from the standard input
-                          stream. File can have both options and data sources
-                          one per line. Contents do not need to be escaped but
-                          spaces in the beginning and end of lines are removed.
-                          Empty lines and lines starting with a hash character
-                          (#) are ignored.
+                          stream. File can have both options and input files
+                          or directories, one per line. Contents do not need to
+                          be escaped but spaces in the beginning and end of
+                          lines are removed. Empty lines and lines starting
+                          with a hash character (#) are ignored.
                           Example file:
                           |  --include regression
                           |  --name Regression Tests
                           |  # This is a comment line
-                          |  my_tests.robot
+                          |  tests.robot
                           |  path/to/test/directory/
                           Examples:
                           --argumentfile argfile.txt --argumentfile STDIN
@@ -364,7 +368,7 @@ creates log file `B.html`. Options accepting no values can be disabled by
 using the same option again with `no` prefix added or dropped. The last option
 has precedence regardless of how many times options are used. For example,
 `--dryrun --dryrun --nodryrun --nostatusrc --statusrc` would not activate the
-dry-run mode and would return normal status rc.
+dry-run mode and would return a normal return code.
 
 Long option format is case-insensitive. For example, --SuiteStatLevel is
 equivalent to but easier to read than --suitestatlevel. Long options can
@@ -400,14 +404,11 @@ $ robot --include smoke --name "Smoke Tests" path/to/tests.robot
 # Executing `robot` module using Python.
 $ python -m robot path/to/tests
 
-# Running `robot` directory with Jython.
-$ jython /opt/robot tests.robot
-
 # Executing multiple test case files and using case-insensitive long options.
 $ robot --SuiteStatLevel 2 --Metadata Version:3 tests/*.robot more/tests.robot
 
 # Setting default options and syslog file before running tests.
-$ export ROBOT_OPTIONS="--critical regression --suitestatlevel 2"
+$ export ROBOT_OPTIONS="--outputdir results --suitestatlevel 2"
 $ export ROBOT_SYSLOG_FILE=/tmp/syslog.txt
 $ robot tests.robot
 """
@@ -416,29 +417,41 @@ $ robot tests.robot
 class RobotFramework(Application):
 
     def __init__(self):
-        Application.__init__(self, USAGE, arg_limits=(1,),
-                             env_options='ROBOT_OPTIONS', logger=LOGGER)
+        Application.__init__(self, USAGE, arg_limits=(1,), env_options='ROBOT_OPTIONS',
+                             logger=LOGGER)
 
     def main(self, datasources, **options):
-        settings = RobotSettings(options)
+        try:
+            settings = RobotSettings(options)
+        except:
+            LOGGER.register_console_logger(stdout=options.get('stdout'),
+                                           stderr=options.get('stderr'))
+            raise
         LOGGER.register_console_logger(**settings.console_output_config)
-        LOGGER.info('Settings:\n%s' % unic(settings))
-        builder = TestSuiteBuilder(settings['SuiteNames'],
-                                   extension=settings.extension,
-                                   rpa=settings.rpa)
+        LOGGER.info(f'Settings:\n{settings}')
+        if settings.pythonpath:
+            sys.path = settings.pythonpath + sys.path
+        builder = TestSuiteBuilder(settings.suite_names,
+                                   included_extensions=settings.extension,
+                                   rpa=settings.rpa,
+                                   lang=settings.languages,
+                                   allow_empty_suite=settings.run_empty_suite)
         suite = builder.build(*datasources)
-        settings.rpa = builder.rpa
-        suite.configure(**settings.suite_config)
+        settings.rpa = suite.rpa
         if settings.pre_run_modifiers:
             suite.visit(ModelModifier(settings.pre_run_modifiers,
                                       settings.run_empty_suite, LOGGER))
+        suite.configure(**settings.suite_config)
         with pyloggingconf.robot_handler_enabled(settings.log_level):
             old_max_error_lines = text.MAX_ERROR_LINES
+            old_max_assign_length = text.MAX_ASSIGN_LENGTH
             text.MAX_ERROR_LINES = settings.max_error_lines
+            text.MAX_ASSIGN_LENGTH = settings.max_assign_length
             try:
                 result = suite.run(settings)
             finally:
                 text.MAX_ERROR_LINES = old_max_error_lines
+                text.MAX_ASSIGN_LENGTH = old_max_assign_length
             LOGGER.info("Tests execution ended. Statistics:\n%s"
                         % result.suite.stat_message)
             if settings.log or settings.report or settings.xunit:
@@ -459,9 +472,9 @@ def run_cli(arguments=None, exit=True):
     """Command line execution entry point for running tests.
 
     :param arguments: Command line options and arguments as a list of strings.
-        Starting from RF 3.1, defaults to ``sys.argv[1:]`` if not given.
+        Defaults to ``sys.argv[1:]`` if not given.
     :param exit: If ``True``, call ``sys.exit`` with the return code denoting
-        execution status, otherwise just return the rc. New in RF 3.0.1.
+        execution status, otherwise just return the rc.
 
     Entry point used when running tests from the command line, but can also
     be used by custom scripts that execute tests. Especially useful if the
@@ -523,9 +536,9 @@ def run(*tests, **options):
     respectively.
 
     A return code is returned similarly as when running on the command line.
-    Zero means that tests were executed and no critical test failed, values up
-    to 250 denote the number of failed critical tests, and values between
-    251-255 are for other statuses documented in the Robot Framework User Guide.
+    Zero means that tests were executed and no test failed, values up to 250
+    denote the number of failed tests, and values between 251-255 are for other
+    statuses documented in the Robot Framework User Guide.
 
     Example::
 
