@@ -15,17 +15,21 @@
 
 import copy
 import os.path
+import os
 
 from robot.output import LOGGER
-from robot.errors import FrameworkError
-from robot.utils import normpath, seq2str2, is_string
+from robot.errors import FrameworkError, DataError
+from robot.utils import normpath, seq2str, seq2str2, is_string
 
 from .builder import ResourceFileBuilder
 from .handlerstore import HandlerStore
 from .testlibraries import TestLibrary
 
 
-class Importer(object):
+RESOURCE_EXTENSIONS = ('.resource', '.robot', '.txt', '.tsv', '.rst', '.rest')
+
+
+class Importer:
 
     def __init__(self):
         self._library_cache = ImportCache()
@@ -48,19 +52,27 @@ class Importer(object):
             LOGGER.info("Imported library '%s' with name '%s'" % (name, alias))
         return lib
 
-    def import_resource(self, path):
+    def import_resource(self, path, lang=None):
+        self._validate_resource_extension(path)
         if path in self._resource_cache:
             LOGGER.info("Found resource file '%s' from cache" % path)
         else:
-            resource = ResourceFileBuilder().build(path)
+            resource = ResourceFileBuilder(lang=lang).build(path)
             self._resource_cache[path] = resource
         return self._resource_cache[path]
 
+    def _validate_resource_extension(self, path):
+        extension = os.path.splitext(path)[1]
+        if extension.lower() not in RESOURCE_EXTENSIONS:
+            raise DataError("Invalid resource file extension '%s'. "
+                            "Supported extensions are %s."
+                            % (extension, seq2str(RESOURCE_EXTENSIONS)))
+
     def _import_library(self, name, positional, named, lib):
         args = positional + ['%s=%s' % arg for arg in named]
         key = (name, positional, named)
         if key in self._library_cache:
-            LOGGER.info("Found test library '%s' with arguments %s from cache"
+            LOGGER.info("Found library '%s' with arguments %s from cache."
                         % (name, seq2str2(args)))
             return self._library_cache[key]
         lib.create_handlers()
@@ -75,22 +87,18 @@ class Importer(object):
                     "(version %s, %s type, %s scope, %d keywords%s)"
                     % (name, seq2str2(args), lib.version or '<unknown>',
                        type, lib.scope, len(lib), listener))
-        if not lib and not lib.has_listener:
+        if not lib:
             LOGGER.warn("Imported library '%s' contains no keywords." % name)
 
     def _copy_library(self, orig, name):
         # This is pretty ugly. Hopefully we can remove cache and copying
         # altogether in 3.0 and always just re-import libraries:
         # https://github.com/robotframework/robotframework/issues/2106
-        # Could then also remove __copy__ methods added to some handlers as
-        # a workaround for this IronPython bug:
-        # https://github.com/IronLanguages/main/issues/1192
         lib = copy.copy(orig)
         lib.name = name
         lib.scope = type(lib.scope)(lib)
         lib.reset_instance()
-        lib.handlers = HandlerStore(orig.handlers.source,
-                                    orig.handlers.source_type)
+        lib.handlers = HandlerStore()
         for handler in orig.handlers._normal.values():
             handler = copy.copy(handler)
             handler.library = lib
@@ -102,7 +110,7 @@ class Importer(object):
         return lib
 
 
-class ImportCache(object):
+class ImportCache:
     """Keeps track on and optionally caches imported items.
 
     Handles paths in keys case-insensitively on case-insensitive OSes.
