@@ -14,26 +14,28 @@
 #  limitations under the License.
 
 import os
-import re
 import sys
 
 
-java_match = re.match(r'java(\d+)\.(\d+)\.(\d+)', sys.platform)
-if java_match:
-    JYTHON = True
-    JAVA_VERSION = tuple(int(i) for i in java_match.groups())
-else:
-    JYTHON = False
-    JAVA_VERSION = (0, 0, 0)
 PY_VERSION = sys.version_info[:3]
-PY2 = PY_VERSION[0] == 2
-PY3 = not PY2
-IRONPYTHON = sys.platform == 'cli'
 PYPY = 'PyPy' in sys.version
 UNIXY = os.sep == '/'
 WINDOWS = not UNIXY
-
 RERAISED_EXCEPTIONS = (KeyboardInterrupt, SystemExit, MemoryError)
-if JYTHON:
-    from java.lang import OutOfMemoryError
-    RERAISED_EXCEPTIONS += (OutOfMemoryError,)
+
+# Part of the deprecated Python 2/3 compatibility layer. For more details see
+# the comment in `utils/__init__.py`. This constant was added to support
+# SSHLibrary: https://github.com/robotframework/SSHLibrary/issues/401
+PY2 = False
+
+
+def isatty(stream):
+    # first check if buffer was detached
+    if hasattr(stream, 'buffer') and stream.buffer is None:
+        return False
+    if not hasattr(stream, 'isatty'):
+        return False
+    try:
+        return stream.isatty()
+    except ValueError:  # Occurs if file is closed.
+        return False
