@@ -14,27 +14,25 @@
 #  limitations under the License.
 
 from robot.errors import DataError
-from robot.model import Keywords
 from robot.output import LOGGER
 from robot.result import Keyword as KeywordResult
-from robot.utils import prepr, unic
-from robot.variables import VariableAssignment, contains_var, is_list_var
+from robot.utils import prepr, safe_str
+from robot.variables import contains_variable, is_list_variable, VariableAssignment
 
-from .steprunner import StepRunner
+from .bodyrunner import BodyRunner
 from .model import Keyword
 from .outputcapture import OutputCapturer
 from .signalhandler import STOP_SIGNAL_MONITOR
 from .statusreporter import StatusReporter
 
 
-class LibraryKeywordRunner(object):
-    _executed_in_dry_run = ('BuiltIn.Import Library',
-                            'BuiltIn.Set Library Search Order')
+class LibraryKeywordRunner:
 
-    def __init__(self, handler, name=None):
+    def __init__(self, handler, name=None, languages=None):
         self._handler = handler
         self.name = name or handler.name
-        self.pre_run_messages = None
+        self.pre_run_messages = ()
+        self.languages = languages
 
     @property
     def library(self):
@@ -48,13 +46,15 @@ class LibraryKeywordRunner(object):
     def longname(self):
         return '%s.%s' % (self.library.name, self.name)
 
-    def run(self, kw, context):
+    def run(self, kw, context, run=True):
         assignment = VariableAssignment(kw.assign)
-        with StatusReporter(context, self._get_result(kw, assignment)):
-            with assignment.assigner(context) as assigner:
-                return_value = self._run(context, kw.args)
-                assigner.assign(return_value)
-                return return_value
+        result = self._get_result(kw, assignment)
+        with StatusReporter(kw, result, context, run):
+            if run:
+                with assignment.assigner(context) as assigner:
+                    return_value = self._run(context, kw.args)
+                    assigner.assign(return_value)
+                    return return_value
 
     def _get_result(self, kw, assignment):
         handler = self._handler
@@ -70,8 +70,9 @@ class LibraryKeywordRunner(object):
         if self.pre_run_messages:
             for message in self.pre_run_messages:
                 context.output.message(message)
-        positional, named = \
-            self._handler.resolve_arguments(args, context.variables)
+        variables = context.variables if not context.dry_run else None
+        positional, named = self._handler.resolve_arguments(args, variables,
+                                                            self.languages)
         context.output.trace(lambda: self._trace_log_args(positional, named))
         runner = self._runner_for(context, self._handler.current_handler(),
                                   positional, dict(named))
@@ -79,7 +80,7 @@ class LibraryKeywordRunner(object):
 
     def _trace_log_args(self, positional, named):
         args = [prepr(arg) for arg in positional]
-        args += ['%s=%s' % (unic(n), prepr(v)) for n, v in named]
+        args += ['%s=%s' % (safe_str(n), prepr(v)) for n, v in named]
         return 'Arguments: [ %s ]' % ' | '.join(args)
 
     def _runner_for(self, context, handler, positional, named):
@@ -109,72 +110,76 @@ class LibraryKeywordRunner(object):
     def dry_run(self, kw, context):
         assignment = VariableAssignment(kw.assign)
         result = self._get_result(kw, assignment)
-        with StatusReporter(context, result, dry_run_lib_kw=True):
+        with StatusReporter(kw, result, context, run=False):
             assignment.validate_assignment()
             self._dry_run(context, kw.args)
 
     def _dry_run(self, context, args):
-        if self._handler.longname in self._executed_in_dry_run:
+        if self._executed_in_dry_run(self._handler):
             self._run(context, args)
         else:
-            self._handler.resolve_arguments(args)
+            self._handler.resolve_arguments(args, languages=self.languages)
+
+    def _executed_in_dry_run(self, handler):
+        keywords_to_execute = ('BuiltIn.Import Library',
+                               'BuiltIn.Set Library Search Order',
+                               'BuiltIn.Set Tags',
+                               'BuiltIn.Remove Tags')
+        return (handler.libname == 'Reserved' or
+                handler.longname in keywords_to_execute)
 
 
 class EmbeddedArgumentsRunner(LibraryKeywordRunner):
 
     def __init__(self, handler, name):
-        LibraryKeywordRunner.__init__(self, handler, name)
-        self._embedded_args = handler.name_regexp.match(name).groups()
+        super().__init__(handler, name)
+        self.embedded_args = handler.embedded.match(name).groups()
 
     def _run(self, context, args):
         if args:
             raise DataError("Positional arguments are not allowed when using "
                             "embedded arguments.")
-        return LibraryKeywordRunner._run(self, context, self._embedded_args)
+        return super()._run(context, self.embedded_args)
 
     def _dry_run(self, context, args):
-        return LibraryKeywordRunner._dry_run(self, context, self._embedded_args)
+        return super()._dry_run(context, self.embedded_args)
+
+    def _get_result(self, kw, assignment):
+        result = super()._get_result(kw, assignment)
+        result.sourcename = self._handler.name
+        return result
 
 
 class RunKeywordRunner(LibraryKeywordRunner):
 
-    def __init__(self, handler, default_dry_run_keywords=False):
-        LibraryKeywordRunner.__init__(self, handler)
-        self._default_dry_run_keywords = default_dry_run_keywords
+    def __init__(self, handler, execute_in_dry_run=False):
+        super().__init__(handler)
+        self.execute_in_dry_run = execute_in_dry_run
 
-    # TODO: Should this be removed altogether?
-    # - Doesn't seem to be really needed.
-    # - Not used with dynamic run kws in the new design (at least currently)
-    def _get_timeout(self, namespace):
+    def _get_timeout(self, context):
+        # These keywords are not affected by timeouts. Keywords they execute are.
         return None
 
     def _run_with_output_captured_and_signal_monitor(self, runner, context):
         return self._run_with_signal_monitoring(runner, context)
 
     def _dry_run(self, context, args):
-        LibraryKeywordRunner._dry_run(self, context, args)
-        keywords = self._get_runnable_dry_run_keywords(args)
-        StepRunner(context).run_steps(keywords)
-
-    def _get_runnable_dry_run_keywords(self, args):
-        keywords = Keywords()
-        for keyword in self._get_dry_run_keywords(args):
-            if contains_var(keyword.name):
-                continue
-            keywords.append(keyword)
-        return keywords
+        super()._dry_run(context, args)
+        keywords = [kw for kw in self._get_dry_run_keywords(args)
+                    if not contains_variable(kw.name)]
+        BodyRunner(context).run(keywords)
 
     def _get_dry_run_keywords(self, args):
+        if not self.execute_in_dry_run:
+            return []
         name = self._handler.name
         if name == 'Run Keyword If':
-            return list(self._get_run_kw_if_keywords(args))
+            return self._get_dry_run_keywords_for_run_keyword_if(args)
         if name == 'Run Keywords':
-            return list(self._get_run_kws_keywords(args))
-        if self._default_dry_run_keywords:
-            return self._get_default_run_kw_keywords(args)
-        return []
+            return self._get_dry_run_keywords_for_run_keyword(args)
+        return self._get_dry_run_keywords_based_on_name_argument(args)
 
-    def _get_run_kw_if_keywords(self, given_args):
+    def _get_dry_run_keywords_for_run_keyword_if(self, given_args):
         for kw_call in self._get_run_kw_if_calls(given_args):
             if kw_call:
                 yield Keyword(name=kw_call[0], args=kw_call[1:])
@@ -189,7 +194,7 @@ class RunKeywordRunner(LibraryKeywordRunner):
             yield else_call
         elif self._validate_kw_call(given_args):
             expr, kw_call = given_args[0], given_args[1:]
-            if not is_list_var(expr):
+            if not is_list_variable(expr):
                 yield kw_call
 
     def _split_run_kw_if_args(self, given_args, control_word, required_after):
@@ -197,18 +202,18 @@ class RunKeywordRunner(LibraryKeywordRunner):
         expr_and_call = given_args[:index]
         remaining = given_args[index+1:]
         if not (self._validate_kw_call(expr_and_call) and
-                    self._validate_kw_call(remaining, required_after)):
+                self._validate_kw_call(remaining, required_after)):
             raise DataError("Invalid 'Run Keyword If' usage.")
-        if is_list_var(expr_and_call[0]):
+        if is_list_variable(expr_and_call[0]):
             return (), remaining
         return expr_and_call[1:], remaining
 
     def _validate_kw_call(self, kw_call, min_length=2):
         if len(kw_call) >= min_length:
             return True
-        return any(is_list_var(item) for item in kw_call)
+        return any(is_list_variable(item) for item in kw_call)
 
-    def _get_run_kws_keywords(self, given_args):
+    def _get_dry_run_keywords_for_run_keyword(self, given_args):
         for kw_call in self._get_run_kws_calls(given_args):
             yield Keyword(name=kw_call[0], args=kw_call[1:])
 
@@ -224,6 +229,6 @@ class RunKeywordRunner(LibraryKeywordRunner):
             if given_args:
                 yield given_args
 
-    def _get_default_run_kw_keywords(self, given_args):
+    def _get_dry_run_keywords_based_on_name_argument(self, given_args):
         index = list(self._handler.arguments.positional).index('name')
         return [Keyword(name=given_args[index], args=given_args[index+1:])]
