@@ -35,41 +35,80 @@ or::
 
 from .argumentparser import ArgumentParser, cmdline2list
 from .application import Application
-from .compat import isatty, py2to3, StringIO, with_metaclass
 from .compress import compress_text
 from .connectioncache import ConnectionCache
 from .dotdict import DotDict
 from .encoding import (CONSOLE_ENCODING, SYSTEM_ENCODING, console_decode,
                        console_encode, system_decode, system_encode)
 from .error import (get_error_message, get_error_details, ErrorDetails)
-from .escaping import escape, unescape, split_from_equals
+from .escaping import escape, glob_escape, unescape, split_from_equals
 from .etreewrapper import ET, ETSource
+from .filereader import FileReader
 from .frange import frange
 from .markuputils import html_format, html_escape, xml_escape, attribute_escape
 from .markupwriters import HtmlWriter, XmlWriter, NullMarkupWriter
 from .importer import Importer
 from .match import eq, Matcher, MultiMatcher
-from .misc import (plural_or_not, printable_name, roundup, seq2str,
-                   seq2str2)
-from .normalizing import lower, normalize, NormalizedDict
-from .platform import (IRONPYTHON, JAVA_VERSION, JYTHON, PY_VERSION,
-                       PY2, PY3, PYPY, UNIXY, WINDOWS, RERAISED_EXCEPTIONS)
+from .misc import (classproperty, isatty, parse_re_flags, plural_or_not,
+                   printable_name, seq2str, seq2str2, test_or_task)
+from .normalizing import normalize, normalize_whitespace, NormalizedDict
+from .platform import PY_VERSION, PYPY, UNIXY, WINDOWS, RERAISED_EXCEPTIONS
 from .recommendations import RecommendationFinder
 from .robotenv import get_env_var, set_env_var, del_env_var, get_env_vars
-from .robotinspect import is_java_init, is_java_method
-from .robotio import binary_file_writer, file_writer
+from .robotinspect import is_init
+from .robotio import binary_file_writer, create_destination_directory, file_writer
 from .robotpath import abspath, find_file, get_link_path, normpath
 from .robottime import (elapsed_time_to_string, format_time, get_elapsed_time,
                         get_time, get_timestamp, secs_to_timestamp,
                         secs_to_timestr, timestamp_to_secs, timestr_to_secs,
                         parse_time)
-from .robottypes import (FALSE_STRINGS, TRUE_STRINGS, is_bytes, is_dict_like,
-                         is_falsy, is_integer, is_list_like, is_number,
-                         is_string, is_truthy, is_unicode, type_name, unicode)
+from .robottypes import (is_bytes, is_dict_like, is_falsy, is_integer, is_list_like,
+                         is_number, is_pathlike, is_string, is_truthy, is_union,
+                         type_name, type_repr, typeddict_types)
 from .setter import setter, SetterAwareType
 from .sortable import Sortable
-from .text import (cut_long_message, format_assign_message,
+from .text import (cut_assign_value, cut_long_message, format_assign_message,
                    get_console_length, getdoc, getshortdoc, pad_console_length,
                    split_tags_from_doc, split_args_from_name_or_path)
-from .unic import prepr, unic
-from .utf8reader import Utf8Reader
+from .unic import prepr, safe_str
+
+
+def read_rest_data(rstfile):
+    from .restreader import read_rest_data
+    return read_rest_data(rstfile)
+
+
+# Quietly deprecated utils. Should be deprecated loudly in RF 7.0.
+# https://github.com/robotframework/robotframework/issues/4501
+
+from .robottypes import FALSE_STRINGS, TRUE_STRINGS
+
+
+# Deprecated Python 2/3 compatibility layer. Not needed by Robot Framework itself
+# after RF 5.0 when Python 2 support was dropped. Should be deprecated loudly in
+# RF 7.0. Notice that there's also `PY2` in the `platform` submodule.
+# https://github.com/robotframework/robotframework/issues/4501
+
+from io import StringIO
+
+
+PY3 = True
+PY2 = JYTHON = IRONPYTHON = False
+is_unicode = is_string
+unicode = str
+unic = safe_str
+roundup = round
+
+
+def py2to3(cls):
+    """Deprecated since RF 5.0. Use Python 3 features directly instead."""
+    if hasattr(cls, '__unicode__'):
+        cls.__str__ = lambda self: self.__unicode__()
+    if hasattr(cls, '__nonzero__'):
+        cls.__bool__ = lambda self: self.__nonzero__()
+    return cls
+
+
+def py3to2(cls):
+    """Deprecated since RF 5.0. Never done anything when used on Python 3."""
+    return cls
