@@ -1,4 +1,5 @@
-#  Copyright 2008-2015 Nokia Solutions and Networks
+#  Copyright 2008-2015 Nokia Networks
+#  Copyright 2016-     Robot Framework Foundation
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -14,13 +15,19 @@
 
 """Implements storing and resolving variables.
 
-This package is mainly for internal usage.
+This package is mainly for internal usage, but utilities for finding
+variables can be used externally as well.
 """
 
 from .assigner import VariableAssignment
-from .isvar import contains_var, is_var, is_scalar_var, is_list_var, is_dict_var
+from .evaluation import evaluate_expression
 from .notfound import variable_not_found
 from .scopes import VariableScopes
-from .splitter import VariableSplitter, VariableIterator
+from .search import (search_variable, contains_variable,
+                     is_variable, is_assign,
+                     is_scalar_variable, is_scalar_assign,
+                     is_dict_variable, is_dict_assign,
+                     is_list_variable, is_list_assign,
+                     VariableIterator)
 from .tablesetter import VariableTableValue, DictVariableTableValue
 from .variables import Variables
